﻿BasePageController('SearchController',
	{
		defaultSearchLocation: 'Locatie wordt bepaald'
	},
	{
        radius: 0,
        map:null,
        markers: [],
		pagebeforeshow: function() {
			// Set default text
			this.element.find(':maf(searchLocation)').val(SearchController.defaultSearchLocation);

			// Start search based on current position
			this.startSearchBasedOnCurrentPosition();
			
			// Select type based on query string.
			if (this.pageData.type == 'dealer') this.pageData.type = 'Dealer';	// correction because the FAQ uses loweracse...
			this.element.find(':maf(searchType)').val(this.pageData.type).selectmenu('refresh', true);
		},
		':maf(searchLocation) focus': function(el) {
				// Reset default value
				el.val('');
		},
		':maf(searchLocation) keypress': function(el, ev) {
			var code = (ev.keyCode ? ev.keyCode : ev.which);
			if (code == 13) { // 13 = Enter keycode
				if (el.val() != SearchController.defaultSearchLocation){
					// Only search if the default value is not listed in the input field.
                    var address = this.element.find(':maf(searchLocation)').val();
                    if (address != null){
                        // Lookup lat/long based on the address.
                        var self = this;
                        var geocoder = new google.maps.Geocoder();
                        var geocoderRequest = { address: address }
                        geocoder.geocode(geocoderRequest, function(results, status) {
                            if (status == google.maps.GeocoderStatus.OK) {
                                if (results != null){
                                    // Take location of first element
                                    var position = results[0].geometry.location;
                                    self.startSearchBasedOnLocation(position.lng(), position.lat());
                                }
                            }
                        });
                    }

				}
			}
		},
		':maf(searchType) change': function(el) { 
			if (el.val() != SearchController.defaultSearchLocation){
                this.pageData.type = this.element.find(':maf(searchType)').val();
				// Only search if the default value is not listed in the input field.
                var latlng = this.map.getCenter();
                this.requestDealers(latlng.lng(), latlng.lat(), this.pageData.type);
			}
		},
        updateSearchBasedOnLocation: function(longitude, latitude) {
            var self = this;
            var geocoder = new google.maps.Geocoder();
            var geocoderRequest = { location: new google.maps.LatLng(latitude, longitude) }
            geocoder.geocode(geocoderRequest, function(results, status) {
                if (status == google.maps.GeocoderStatus.OK) {
                    if (results != null){
                        // Take first element
                        //var address = results[0].formatted_address;
                        var address = '';
                        var found = false;

                        $.each(results[0].address_components, function(i, address_component) {
                            if (address_component.types[0] == 'locality') {
                                address = address_component.long_name;
                            }
                        });


                        self.element.find(':maf(searchLocation)').val(address);
                    }
                }
            });

            // Default search, based on GPS position and query string parameter
            self.requestDealers(longitude, latitude, self.pageData.type);

        },
        startSearchBasedOnLocation:function(longitude, latitude) {
            this.createMap(longitude, latitude);
        },
		startSearchBasedOnCurrentPosition: function() {
			// Get current position
			var self = this;
			navigator.geolocation.getCurrentPosition(function(position) {
                 self.startSearchBasedOnLocation(position.coords.longitude, position.coords.latitude);
			},
			function(error){
				self.showError('De huidige locatie kon niet bepaald worden.');
			},
			{
				// Options: required for Android emulator.
				enableHighAccuracy: true
			});
		},
		requestDealers: function(longitude, latitude, type) {
			l('requestDealers: ' + type);
			
			// Show loading message and disable input
			$.mobile.loadingMessage = 'Kaart wordt geladen';
			$.mobile.showPageLoadingMsg();
			this.element.find(':maf(searchLocation)').prop('disabled', true);
			this.element.find(':maf(searchType)').prop('disabled', true);

			var self = this;
			var carData = Contract.getActiveContract();
			if (carData != null){
                var params;
                if (type=='Dealer') {
                    params = $.extend({
                        lat: latitude,
                        lng: longitude,
                        type: type,
                        brand: carData.codemerk,
                        radius: self.radius,
                        limit: 50
                    }, Settings.searchParams);
                } else {
                    // Geen dealers, niet filteren op merk nu
                    params = $.extend({
                        lat: latitude,
                        lng: longitude,
                        type: type,
                        radius: self.radius,
                        limit: 50
                    }, Settings.searchParams);
                }
				$.ajax({
					url: Settings.searchUrl,
					data: $.param(params),
					type: 'GET',
					dataType: 'json',
					timeout: Settings.ajaxTimeout,
					success: function(data){
						if (data != null){
                            // Enable input, hide loading, show map.
                            self.element.find(':maf(searchLocation)').prop('disabled', false);
                            self.element.find(':maf(searchType)').prop('disabled', false);
                            $.mobile.hidePageLoadingMsg();
                            self.showMarkers(longitude, latitude, data);
						}
					},
					error: function(xhr, textStatus, errorThrown){
						$.mobile.hidePageLoadingMsg();
						self.showError('De adresgegevens konden niet worden opgehaald.');
					}
				});
			}
		},
        createMap:function(longitude,latitude) {
            var firstTime = true;
            var self = this;
            if (!this.map) {
                // Bounds are used to zoom out to show all markers in the map.

                // Current position
                var myLatLong = new google.maps.LatLng(latitude, longitude);
                // Options, do not show any controls, show road map.
                var options = {
                    mapTypeId: google.maps.MapTypeId.ROADMAP,
                    zoom: 10,
                    panControl: false,
                    zoomControl: true,
                    mapTypeControl: false,
                    scaleControl: false,
                    streetViewControl: false,
                    overviewMapControl: false
                }
                // Create map

                this.map = new google.maps.Map(document.getElementById('searchMap'), options);

                google.maps.event.addListener(this.map, 'bounds_changed', function() {
                    function finished(){
                        var location = self.map.getCenter();
                        //var bounds = new google.maps.LatLngBounds();
                        var bounds = self.map.getBounds();
                        var ne = bounds.getNorthEast();

                        var radiusm = google.maps.geometry.spherical.computeDistanceBetween(location, ne);
                        var radiuskm = Math.ceil(radiusm / 1000);
                        self.radius = radiuskm;

                        var location = self.map.getCenter();
                        self.updateSearchBasedOnLocation(location.lng(), location.lat());
                    }
                    clearTimeout(this.timeout);
                    this.timeout = setTimeout(finished, 200);
                });


                this.map.setCenter(new google.maps.LatLng(latitude, longitude));

            } else {
                this.map.setCenter(new google.maps.LatLng(latitude, longitude));
                firstTime = false;
            }


        },
		showMarkers: function(longitude, latitude, data) {
			l('showMarkers');
            var self = this;

            while (self.markers.length > 0) {
                var tmpMarker = self.markers.pop();
                tmpMarker.setMap(null);
            }

			// Loop the dealer information.
            var curInfoWindow = null;
			$.each(data['dealer-array'], function(i, dealer){
					
				// Create latlong
				var latLong = new google.maps.LatLng(dealer.lat, dealer.lng);
						
				// Add marker to map
				var marker = new google.maps.Marker({ map: self.map, position: latLong, title: dealer.name, icon: 'img/gmap_icon.png' });
                self.markers.push(marker);

				// Add event to marker to show info upon click
				google.maps.event.addListener(marker, 'click', function() {
					// Show info window.
					Navigation.gotoPage('searchresult', { dealer: dealer });

                });

			});
			


		}
	}
);