﻿$.Class('Navigation', {
	currentIndex: undefined,
	history: [],
	root: null,
	init: function() {
		// initialize navigation when the dom is ready
		$(function() {
			Navigation.initialize();
		});
		var self = this;
		$(document).bind('deviceready', function() {
			l('Device ready: platform: ' + device.platform + ', name: ' + device.name + ', version: ' + device.version + ', cordova/phonegap: ' + (device.cordova || device.phonegap));
			$(document).bind('backbutton', self.proxy(self._backButtonHandler));
		});
	},
	initialize: function() {
		// determine root
		this.root = location.href.match(/^(.*\/).*$/) && RegExp.$1;
		// attach all controllers
		this._attachControllers(document);
		// determine first page and controller
		var page = $('[data-role="page"]:first');	// we cannot use :jqmData or $.mobile.ns because jquery mobile has not loaded yet
		var controller = page.controller(BasePageController);
		// add first page to history
		var item = { index: 0, pageId: page.attr('id'), data: {}, context: {} };
		this.history.push(item);
		this.currentIndex = 0;
		// set data and context on controller
		if (controller) {
			controller.pageData = item.data;
			controller.pageContext = item.context;
			controller.pageReshow = false;
		}
		// catch popstate event to detect browser back/forward buttons
		$(window).bind('popstate', this.proxy(this._popStateHandler));
	},
	_backButtonHandler: function() {
		// handle device backbutton

        var curItem = this.getCurrentItem();
        var self = this;
        this._ensurePageLoaded(curItem.pageId, function() {
                // set the page data for this item, also get controller for other page
                var controller = self._setPageDataForItem(curItem, null, false);

                if (typeof(controller.onBack) == 'function') {
                    controller.onBack();
                } else {
                    self.gotoPreviousPage();
                }
        });
	},
	_loadScripts: function(scripts) {
		if (!scripts || scripts.length == 0) return;
		var head = $('head');
		var self = this;
		if (!this._scripts) {
			this._scripts = {};
			head.find('script[src]').each(function() {
				self._scripts[this.src] = true;
			});
		}
		scripts.each(function() {
			if (!self._scripts[this.src]) {
				// script not present yet: add it
				head.append($('<script/>', { type: this.type, src: this.src }));
				self._scripts[this.src] = true;
			}
		});
	},
	_attachControllers: function(ancestor) {
		$(ancestor).find('[data-maf-controller]').andSelf().each(function() {
			var element = $(this);
			var controllerNames = element.data('mafController');
			if (controllerNames) {
				controllerNames = controllerNames.split(' ');
				for (var i = 0; i < controllerNames.length; i++) {
					var controllerName = controllerNames[i];
					if (controllerName) {
						if (element[controllerName]) {
							element[controllerName]();
						} else {
							l('Warning: no controller "' + controllerName + '" found');
						}
					}
				}
			}
		});
	},
	_popStateHandler: function() {
		// determine pageId from hash or href
		var pageId;
		if (location.hash && location.hash.indexOf('#') == 0) {
			// remove # from hash to get pageid
			pageId = location.hash.substr(1);
			if (pageId.indexOf('/') == 0) {
				// hash like '#/otherpage.html', will re replaced by normal url witout has, so ignore
				return;
			}
		}
		if (pageId && pageId.indexOf($.mobile.dialogHashKey) != -1) {
			// a dialog is shown: no need to put this in the history
			return;
		}
		if (!pageId) {
			pageId = this._urlToPageId(location.href);
		}
		// find closest corresponding page in history (first current, then previous, then next pages)
		var foundItem = this._findPageIdInHistory(pageId);
		if (!foundItem) {
			// hash not in history, probably somebody typed it in the browser addressbar, just add it to history
			var item = { pageId: pageId, data: {}, context: {} };
			this._addItemToHistory(item);
		} else if (foundItem.index === this.currentIndex) {
			// our history corresponds with browser: no further action required
			return;
		} else {
			// our history does not correspond with browser: back or previous button was used, or link with data-rel="back"
			this.currentIndex = foundItem.index;
			this._setPageDataForItem(foundItem, null, true);
		}
	},
	_findPageIdInHistory: function(pageId) {
		// first chack current page, then previous pages
		for (var i = this.currentIndex; i >= 0; i--) {
			if (this.history[i].pageId === pageId) {
				return this.history[i];
			}
		}
		// if not found then check next pages
		for (var i = this.currentIndex + 1; i < this.history.length; i++) {
			if (this.history[i].pageId === pageId) {
				return this.history[i];
			}
		}
	},
	gotoPage: function(pageId, data, extraContext, options) {
		var currentItem = this.getCurrentItem();
		
		// ensure any slashes are replaced by underscores since slashes are not allowed in id attributes
		pageId = pageId.replace(/\/|\\/g, '_');
		
		// check if we are navigating to other page
		if (currentItem.pageId === pageId) {
			var settings = $.extend({}, $.mobile.changePage.defaults, options);
			if (!settings.allowSamePageTransition) return;	// allready on right page
		}
		
		var self = this;
		this._ensurePageLoaded(pageId, function() {
			// create item for history
			data = data || {};
			var context = $.extend({}, currentItem.context, extraContext);
			var item = { pageId: pageId, data: data, context: context };
			// set the page data for this item, also get controller for previous page
			var controller = self._setPageDataForItem(item, null, false);
			// the action to perform when changing to the new page
			var action = function() {
                //var addToHistory = $.id(pageId).attr('data-maf-putinhistory');

                //if ((!addToHistory) || (addToHistory != 'false')){
    				// add new item to history
	    			self._addItemToHistory(item);
                //}
				// change to new page
				$.mobile.changePage($.id(pageId), options);
			};
			// load data and display page
			self._loadDataAndDisplayPage(controller, action);
		});
	},
	_pageIdToUrl: function(pageId) {
		// replace _ by / and add extension
		// start with root so relative location of current page does not matter
		var url = pageId.replace(/_/g, '/');
		return this.root + url + '.html';
	},
	_urlToPageId: function(url) {
		var relativeUrl = url.substr(this.root.length);
		if (relativeUrl == 'index.html') {
			// no hash and on index.html: first page
			return $(':jqmData(role="page"):first').attr('id');
		}
		relativeUrl = relativeUrl.substr(0, relativeUrl.lastIndexOf('.'));
		return relativeUrl.replace(/\//g, '_');
	},
	_ensurePageLoaded: function(pageId, pageLoadedCallback) {
		// check if pageid is present in dom, otherwise load it as url
		if ($.id(pageId).length == 0) {
			//HACK: hook into jQuery.find, the first call to this method will be to find the first page in the loaded html. We use it to also get any scripts present in the loaded html
			var scripts;
			var findOriginal = $.fn.find;
			$.fn.find = function(selector) {
				if (this.closest('body').length == 0) {
					// reset hook
					$.fn.find = findOriginal;
					// not in a body, should be the loaded html: find scripts
					scripts = this.children('script[src]');
				}
				return findOriginal.apply(this, [selector]);
			};
			
			// wait for pagebeforecreate event to attach controllers
			var self = this;
			var pagebeforecreateHandler = function(ev) {
				// load scripts (if any)
				self._loadScripts(scripts);
				// ensure correct id
				ev.target.id = pageId;
				// attach all controllers
				self._attachControllers(ev.target);
			};
			$(document).bind('pagebeforecreate', pagebeforecreateHandler);
			var url = this._pageIdToUrl(pageId);
			$.mobile.loadPage(url, { showLoadMsg: false }).always(function() {
				// unbind the event handler
				$(document).unbind('pagebeforecreate', pagebeforecreateHandler);
				// reset hook
				$.fn.find = findOriginal;
			}).done(function(absUrl, options, page) {
				// change the page
				pageLoadedCallback();
			}).fail(function(absUrl) {
				l('Could not load page: ' + absUrl);
			});
		} else {
			pageLoadedCallback();
		}
	},
	_loadDataAndDisplayPage: function(controller, action) {
		if (controller) {
			// do custom display action if necessary
			var isCustomDisplay = controller.customDisplay();
			// in case of custom display (such as captureVideo) the history is not altered as we do not know when a user returns
			// and the user cannot continue to other pages of the app so for the app we remain on the same page...
			if (isCustomDisplay) return;
			// show default loading message
			$.mobile.showPageLoadingMsg();
			// loadData aanroepen, deze moet de callback uitvoeren
			controller.loadData(action);
		} else {
			// no controller: change to (static) page immediately
			action();
		}
	},
	gotoPreviousPage: function(extraData, withDataReload) {
		this.gotoPageInHistory(-1, extraData, withDataReload);
	},
	gotoPageOrPrevious: function(pageId, data, withDataReload) {
		var previousItem = this.getRelativeItem(-1);
		if (previousItem && previousItem.pageId === pageId) {
			this.gotoPreviousPage(data, withDataReload);
		} else {
			this.gotoPage(pageId, data);
		}
	},
	gotoFollowingPage: function(extraData, withDataReload) {
		this.gotoPageInHistory(1, extraData, withDataReload);
	},
	gotoPageOrFollowing: function(pageId, data, withDataReload) {
		var followingItem = this.getRelativeItem(1);
		if (followingItem && followingItem.pageId === pageId) {
			this.gotoFollowingPage(data, withDataReload);
		} else {
			this.gotoPage(pageId, data);
		}
	},
	gotoPageInHistory: function(delta, extraData, withDataReload) {
		if (!delta) return;
		// get other history item
		var otherItem = this.getRelativeItem(delta);
		if (!otherItem) return;
		var self = this;
		this._ensurePageLoaded(otherItem.pageId, function() {
			// set the page data for this item, also get controller for other page
			var controller = self._setPageDataForItem(otherItem, extraData, !withDataReload);
			// check if it is a same-page transition
			var correctedDelta = self._correctDelta(delta);
			var isSamePageTransition = correctedDelta == 0;
			// the action to perform when changing to the previous page
			var action = function() {
				// set previous page as current
				self.currentIndex += delta;
				// go back to previous page
                //Siep: TODO
                /*
                if (isSamePageTransition) {
                    //HACK: fool jquery mobile to think that we are navigating back because of hash change, it will do the navigation for us in the correct way
                    var directHashChangeOriginal = $.mobile.urlHistory.directHashChange;
                    $.mobile.urlHistory.directHashChange = function(opts) { this.activeIndex += delta; (opts.either || opts.isBack)(true); };
                    $.mobile.changePage(self.getCurrentPage(), { fromHashChange: true, allowSamePageTransition: true });
                    $.mobile.urlHistory.directHashChange = directHashChangeOriginal;
                } else {
                    //HACK: fool jquery mobile to think that we are navigating back because of hash change, it will do the navigation for us in the correct way
                    var directHashChangeOriginal = $.mobile.urlHistory.directHashChange;
                    $.mobile.urlHistory.directHashChange = function(opts) { this.activeIndex += delta; (opts.either || opts.isBack)(true); };
                    $.mobile.changePage($.id(otherItem.pageId), { fromHashChange: true, allowSamePageTransition: false });
                    $.mobile.urlHistory.directHashChange = directHashChangeOriginal;
                }*/


				if (isSamePageTransition) {
					//HACK: fool jquery mobile to think that we are navigating back because of hash change, it will do the navigation for us in the correct way
					var directHashChangeOriginal = $.mobile.urlHistory.directHashChange;
					$.mobile.urlHistory.directHashChange = function(opts) { this.activeIndex += delta; (opts.either || opts.isBack)(true); };
					$.mobile.changePage(self.getCurrentPage(), { fromHashChange: true, allowSamePageTransition: true });
					$.mobile.urlHistory.directHashChange = directHashChangeOriginal;
				} else {
					window.history.go(correctedDelta);
				}
			};
			if (withDataReload) {
				// load data and display previous page
				self._loadDataAndDisplayPage(controller, action);
			} else {
				// display previous page
				action();
			}
		});
	},
	_correctDelta: function(delta) {
		// correct the specified delta to the number of steps the browser history has to go back/forward
		// this can be different due to same-page transitions (which are not present in the browser history)
		var correctedDelta = 0;
		var deltaIndex = this.currentIndex + delta;
		var prevPageId = this.getCurrentPageId();
		if (delta < 0) {
			for (var d = -1; d >= delta; d--) {
				var pageId = this.getRelativeItem(d).pageId;
				if (pageId !== prevPageId) correctedDelta--;
				prevPageId = pageId;
			}
		} else {
			for (var d = 1; d <= delta; d++) {
				var pageId = this.getRelativeItem(d).pageId;
				if (pageId !== prevPageId) correctedDelta++;
				prevPageId = pageId;
			}
		}

        //Siep: nieuw: skip 'onzichtbare' pagina's in de history. Deze zijn gemarkeerd als niet aanwezig.
        var index = correctedDelta;
        var item = this.getRelativeItem(index);

        while (item && item.invisibleHistoryItem) {
            if (delta >= 0) {
                index++;
                correctedDelta++;
            } else {
                index--;
                correctedDelta--;
            }
            item = this.getRelativeItem(index);
        }


		return correctedDelta;
	},
	getCurrentItem: function() {
		// get current item from history
		return this.history[this.currentIndex];
	},
	getCurrentPageId: function() {
		// determine current pageId
		return this.getCurrentItem().pageId;
	},
	getCurrentPage: function() {
		// get current pageId
		var pageId = this.getCurrentPageId();
		// determine page
		return $.id(pageId);
	},
	getRelativeItem: function(delta) {
		// get other item from history, if not present then <undefined> will be returned
        /*
        var index = this.currentIndex + delta;
        var item = this.history[index];

        while ((item) && (item.invisibleHistoryItem)) {
            if (delta >= 0) {
                index++;
            } else {
                index--;
            }

            item = this.history[index];
        }

        return item;
        */
		return this.history[this.currentIndex + delta];
	},
	_setPageDataForItem: function(item, extraData, reshow) {
		// determine page and pagecontroller
		var page = $.id(item.pageId);
		var controller = page.controller(BasePageController);
		// add extraData if present
		if (extraData) $.extend(item.data, extraData);
		// set data and context on controller
		if (controller) {
			controller.pageData = item.data;
			controller.pageContext = item.context;
			controller.pageReshow = reshow;
		}
		return controller;
	},
	_addItemToHistory: function(item) {
		// clip of part of history after currentIndex
        var addToHistory = $.id(item.pageId).attr('data-maf-putinhistory');

        if ((!addToHistory) || (addToHistory != 'false')){
            item.invisibleHistoryItem = false;
        } else {
            item.invisibleHistoryItem = true;
        }

    	this.history = this.history.slice(0, this.currentIndex + 1);
	    // add new page to history
	    var newLength = this.history.push(item);
	    item.index = this.currentIndex = newLength - 1;

	}
}, {
	init: function() {
		$.error('Navigation class should not be instantiated')
	}
});
